////////////////
//Script to autostart page video 
// and handle reload of stalled and off air videos
///////////////

console.debug('VideoPage init');


var DEBUG_BOX;
//configuration
if(typeof SHOW_DEBUG_BOX=='undefined')
{
	DEBUG_BOX=true; //show debug box with status
}
else
{
	DEBUG_BOX = SHOW_DEBUG_BOX; //externally set 
}
var LOOCKUP_TIMEOUT=300*1000; //timeout to reload page if video not found on lookup
var RESTART_BUFFERING_TIMEOUT=40*1000; //timeout to reload video when stalled
var RELOAD_NOPLAY_TIMEOUT=300*1000; //timeout to reload page when stalled

//lookup conf
var LOOCKUP_INITIAL_INTERVAL=500; //lookup initial frequency
var LOOCKUP_INITIAL_INTERVAL_DURATION = 20*500; //initial lookup count
var LOOCKUP_DELAYED_INTERVAL=1000; //reduced lookup frequency after LOOCKUP_INITIAL_INTERVAL_COUNT iterations

//lookup variables
var lookupVideoInterval = LOOCKUP_INITIAL_INTERVAL;// will be reduced to LOOCKUP_DELAYED_INTERVAL after LOOCKUP_INITIAL_INTERVAL_COUNT
var lookupStartTime;
var locateTimeoutId = 0;
var testTimeoutId = 0;

if(DEBUG_BOX)
	createDebugBox();

var coverBox = null;

/**
 * Iteratively Locates video element inside page 
 *   when found invokes onVideoFound
 *   after LOOCKUP_INITIAL_INTERVAL_COUNT reduces iterations to LOOCKUP_DELAYED_INTERVAL
 *   after reaching LOOKUP_TIMEOUT_COUNT reloads the page
 */
function videoPageLocateVideo()
{			
//	if(coverBox==null && document.body!=null)
//	{
//		coverBox = document.createElement("div");
//		coverBox.style.position = "absolute";
//		coverBox.style.backgroundColor = "black";
//	}
	document.body.style.visibility = "hidden";
	var curTime = new Date().getTime();
	console.debug('VideoPage locating video');
	var videos=document.getElementsByTagName('VIDEO');
	if(videos && videos.length)
	{
		//video found
		var video=videos[videos.length-1];
		onVideoFound(video);				
	}
	else
	{
		var timeDelta = curTime - lookupStartTime;	
		var fcnTime = new Date().getTime();
		if(timeDelta < LOOCKUP_TIMEOUT)
		{
			if(timeDelta>LOOCKUP_INITIAL_INTERVAL_DURATION && 
					lookupVideoInterval!=LOOCKUP_DELAYED_INTERVAL )
			{
				lookupVideoInterval = LOOCKUP_DELAYED_INTERVAL;
			}			
			var remains = (LOOCKUP_TIMEOUT-timeDelta)/1000 | 0;
			if(DEBUG_BOX)
			{
				statusBox.innerText = "";
				statusBox.innerText = "Waiting for video (reload in " + remains + " sec)";
			}
			console.trace('VideoPage no video yet  ' + " (reload in " + remains + " sec) took " + (new Date().getTime()-fcnTime));
			//call next iteration
			if(locateTimeoutId)
				clearTimeout(locateTimeoutId);
			locateTimeoutId = setTimeout(videoPageLocateVideo, lookupVideoInterval);
		}
		else
		{
			console.trace('VideoPage no video found, reloading page');
			if(DEBUG_BOX)
			{
				statusBox.innerText = "No video, reloading page";
			}
			//timeed out - reload
			locateTimeoutId = setTimeout(reloadPage, 1000);
		}		
	}
}
//start locating video on script load
lookupStartTime = new Date().getTime();
videoPageLocateVideo();


//video status monitoring variables
var currentVideo;
var pageStartTime = 0;
var lastPlayPos = 0;
var lastPlayPosChangeTime=0;
var reloadCount = 0;
var bufferingCount = 0;
var bufferingUpdateTime = 0;
var videoTestActive = true;
var lastPlayActiveTime = 0;
var disableRebuffer = false;

/**
 * Called by player thread (direct JS video.play() does not work on 4.1)
 */
function __onPlayVideo()
{
	document.body.style.visibility = "visible";
	currentVideo.play();
	if(DEBUG_BOX)
	{
		statusBox.innerText = "Start playing";
		//testBox.style.opacity = "0.7";
	}
	resetTestTimeout();
	testTimeoutId = setTimeout(onVideoTest, 500);	
}

function __onReloadVideo()
{
	currentVideo.pause();
	currentVideo.load();
	__onPlayVideo();
}

/**
 * Plays the video
 * Invokes player thread to call back __onPlayVideo
 */
function playVideo()
{
	nvPageHandler.invokePlayVideo();
}

function reloadVideo()
{
	nvPageHandler.invokeReloadVideo();
}


/**
 * Reloads the page
 */
function reloadPage()
{
	pageStartTime = 0;
	videoTestActive = false;
	location.reload();
}


/**
 * Called when video is found and starts monitoring play status
 * @param video
 */
function onVideoFound(video)
{
	console.trace('VideoPage video found, starting ' + video.currentSrc);				
	if(DEBUG_BOX)
	{
		statusBox.innerText = "Video found, starting";
	}			
	currentVideo = video;
	pageStartTime = new Date().getTime();
	lastPlayPosChangeTime = new Date().getTime();
	lastPlayActiveTime = lastPlayPosChangeTime;
	playVideo();
}

function resetTestTimeout()
{
	if(testTimeoutId!=0)
	{
		clearTimeout(testTimeoutId);
		testTimeoutId = 0;
	}
}

/**
 * Monitors video status iteratively
 * If not playing for RESTART_BUFFERING_TIMEOUT, restart the video
 * If not playing for RELOAD_NOPLAY_TIMEOUT, reload the page
 * If DEBUG_BOX is true, updates status display
 */
function onVideoTest()
{
	resetTestTimeout();
	if(!videoTestActive)
		return;
	var newPlayPos = currentVideo.currentTime;
	var playstatus = "";
	var curTime = new Date().getTime();
	var scheduleNewTest = true;
	
	if(newPlayPos!=lastPlayPos)
	{
		lastPlayPos = newPlayPos;
		lastPlayPosChangeTime = curTime;
		lastPlayActiveTime = lastPlayPosChangeTime;
		playStatus = "playing";
	}
	else
	{
		if(curTime-lastPlayActiveTime < RELOAD_NOPLAY_TIMEOUT)
		{
			if(curTime-bufferingUpdateTime > RESTART_BUFFERING_TIMEOUT)
			{
				bufferingCount++;
				bufferingUpdateTime = curTime;
			}
			var changeDif = curTime-lastPlayPosChangeTime;
			if(changeDif<10000)
			{
				playStatus = "buffering";
			}
			else
			{
				if(changeDif<RESTART_BUFFERING_TIMEOUT)
				{
					if(DEBUG_BOX)
					{
						playStatus = "stalled, reload video in " + 
							 ((RESTART_BUFFERING_TIMEOUT-changeDif)/1000 | 0) + " sec, " +
							"page " + ((RELOAD_NOPLAY_TIMEOUT-curTime+lastPlayActiveTime)/1000 | 0) + " sec"
							;
					}
					else
					{
						playStatus = "stalled";
					}
				}
				else
				{
					playStatus = "reload";
					reloadCount++;
					lastPlayPosChangeTime = curTime;
					if(!disableRebuffer)
					{
						//document.body.style.visibility = "hidden";
						scheduleNewTest = false;
						reloadVideo();
					}
					else
					{
						currentVideo.pause();
					}
				}			
			}
		}
		else
		{
			scheduleNewTest = false;
			reloadPage();
		}
	}
	if(DEBUG_BOX)
		updateDebugBox(playStatus);
	if(scheduleNewTest)
	{
		testTimeoutId = setTimeout(onVideoTest, 1000);
	}
}

/**
 * Updates status display
 * @param playStatus
 */
function updateDebugBox(playStatus)
{
	var pageUptime = new Date();
	pageUptime.setTime(pageUptime.getTime() - pageStartTime);
	statusBox.innerHTML = 
		new Date().timeNow() +
		" page uptime " + pageUptime.utcTimeNow() +
		"<br/><br/>" + 
		" status: " + playStatus +
		"<br/><br/>" +
		" video pos=" + currentVideo.currentTime.toFixed(2) + " ended=" + currentVideo.ended +
		"<br/><br/>" +
		" error=" + currentVideo.error + 
		"<br/><br/>" +
		" stats:  bufferring:" + bufferingCount + " vid reload:" + reloadCount + " page reload: " + window.nvPageReloadCount; 
	
}


//debug status display box  
var testBox;
function createDebugBox()
{
	testBox = document.createElement("div");
	testBox.style.backgroundColor = "yellow";
	testBox.style.color = "darkblue";
	testBox.style.paddingTop = "10px";
	testBox.style.position = "absolute";
	testBox.style.left = "0";
	testBox.style.top = "20px";
	testBox.style.minWidth = "300px" 
	testBox.style.paddingTop = "5px" 
	testBox.style.paddingBottom = "30px" 
	testBox.style.paddingLeft = "10px" 
	testBox.style.paddingRight = "10px" 
	testBox.style.zIndex = "1000";
	
	var button = document.createElement("BUTTON");
	button.innerText = "Reload";
	button.style.color="black";
	button.onclick = function(){
		if(currentVideo!=null)
		{
			resetTestTimeout();
			reloadVideo();
		}
		else
		{
			console.debug("reload click - no video");
		}
	};
	testBox.appendChild(button);
	var button = document.createElement("BUTTON");
	button.innerText = "Reload Page";
	button.style.color="black";
	button.onclick = function(){
		console.debug("reload page click");
		reloadPage();
	};
	testBox.appendChild(button);
	
	var button = document.createElement("BUTTON");
	button.innerText = "Simulate Stalled";
	button.style.color="black";
	button.onclick = function(){
		console.debug("stalled click");
		if(currentVideo)
			currentVideo.pause();
	};
	testBox.appendChild(button);
	
	var button = document.createElement("BUTTON");
	button.innerText = "Simulate Broken";
	button.style.color="black";
	button.onclick = function(){
		console.debug("broken click");
		disableRebuffer = true;
		if(currentVideo)
			currentVideo.pause();
	};
	testBox.appendChild(button);
	
	
	
	testBox.appendChild( document.createElement("BR") );
	testBox.appendChild( document.createElement("BR") );
	
	statusBox = document.createElement("div");
	statusBox.style.fontWeight = "bold";
	statusBox.style.fontSize = "22px";
	statusBox.innerText = "Loading...";
	
	testBox.appendChild(statusBox);
	document.body.appendChild(testBox);
	
	//add to date current time label shortcuts

	Date.prototype.timeNow = function () {
	    return ((this.getHours() < 10)?"0":"") + this.getHours() +":"+ ((this.getMinutes() < 10)?"0":"") + this.getMinutes() +":"+ ((this.getSeconds() < 10)?"0":"") + this.getSeconds();
	}

	Date.prototype.utcTimeNow = function () {
	   return ((this.getUTCHours() < 10)?"0":"") + this.getUTCHours() +":"+ ((this.getUTCMinutes() < 10)?"0":"") + this.getUTCMinutes() +":"+ ((this.getUTCSeconds() < 10)?"0":"") + this.getUTCSeconds();
	}
	
}

